from _typeshed import Incomplete
from amsdal_data.connections.errors import AmsdalConnectionError as AmsdalConnectionError
from amsdal_data.table_schemas.constants import PRIMARY_PARTITION_KEY as PRIMARY_PARTITION_KEY, SECONDARY_PARTITION_KEY as SECONDARY_PARTITION_KEY
from amsdal_utils.query.data_models.filter import Filter as Filter
from amsdal_utils.query.data_models.order_by import OrderBy as OrderBy
from amsdal_utils.query.data_models.paginator import CursorPaginator, NumberPaginator
from amsdal_utils.query.data_models.query_specifier import QuerySpecifier as QuerySpecifier
from amsdal_utils.query.enums import Lookup
from amsdal_utils.query.utils import Q
from collections.abc import Callable as Callable
from pydantic import BaseModel
from typing import Any, ClassVar

SUPPORTED_NESTED_FIELDS: Incomplete
ADDRESS_FIELD: str
MODEL_TABLE_ALIAS: str
METADATA_TABLE_ALIAS: str

class NoValue: ...

class SqlOperatorTemplate(BaseModel):
    build_statement: ClassVar[Callable[[Any, str, Any, str], str]]
    build_value: ClassVar[Callable[[Any, Any], Any]]
    template: str | Callable[[Any], str]
    value_template: str | None
    value_modifier: Callable[[Any], Any] | None
    def build_statement(self, field_name: str, value: Any, table_name: str = '') -> str: ...
    def build_value(self, value: Any) -> Any: ...

sql_operator_map: dict[Lookup, SqlOperatorTemplate]
METADATA_SELECT_STATEMENT: str
METADATA_SELECT_FIELD: Incomplete

class SqliteStatementsMixin:
    def _build_insert_statement(self, table_name: str, data: dict[str, Any]) -> tuple[str, tuple[Any, ...]]: ...
    def _build_delete_statement(self, table_name: str, data: dict[str, Any]) -> tuple[str, tuple[Any, ...]]: ...
    def _build_select_statement(self, table_name: str, query_specifier: QuerySpecifier | None = None, conditions: Q | None = None, pagination: NumberPaginator | CursorPaginator | None = None, order_by: list[OrderBy] | None = None, *, use_internal_meta_tables: bool = False) -> tuple[str, list[Any]]: ...
    def __select_statement(self, table_name: str, select_only: str, conditions: Q | None = None, pagination: NumberPaginator | CursorPaginator | None = None, order_by: list[OrderBy] | None = None, *, is_internal_table: bool = False) -> tuple[str, list[Any]]: ...
    def __select_with_internal_meta_statement(self, table_name: str, select_only: str, conditions: Q | None = None, pagination: NumberPaginator | CursorPaginator | None = None, order_by: list[OrderBy] | None = None) -> tuple[str, list[Any]]: ...
    def _build_count_statement(self, table_name: str, conditions: Q | None = None, *, use_internal_meta_tables: bool = False) -> tuple[str, list[Any]]: ...
    def _get_conditions_statement(self, conditions: Q, *, is_internal_table: bool = False, use_internal_meta_tables: bool = False, is_grouped: bool = False) -> tuple[str, list[Any]]: ...
    def _get_filter_statement(self, filter_item: Filter, *, is_internal_table: bool = False, use_internal_meta_tables: bool = False) -> tuple[str, Any]: ...
    def _nested_filter_statement(self, field_name: str, nested_fields: str, sql_operator_template: SqlOperatorTemplate, value: Any, value_type: Any, *, is_internal_table: bool = False, use_internal_meta_tables: bool = False) -> tuple[str, Any]: ...
    @staticmethod
    def _resolve_nested_field_name(field_name: str, nested_fields: str, *, value_type: Any = ..., is_internal_table: bool = False, use_internal_meta_tables: bool = False) -> str: ...
    def _build_order_by_statement(self, order_by: list[OrderBy] | None, *, is_internal_table: bool = False, use_internal_meta_tables: bool = False) -> str: ...
    @classmethod
    def _to_sql_value(cls, value: Any) -> Any: ...
    def _describe_table(self, table_name: str) -> dict[str, str]: ...
    def _process_result(self, result: dict[str, Any], table_description: dict[str, str]) -> dict[str, Any]: ...
    @staticmethod
    def _process_field(field_type: str, value: Any) -> Any: ...
